/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 * <p>
 * http://www.foxitsoftware.com
 * <p>
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
package com.foxit.uiextensions.annots.ink;

import android.content.Context;
import android.content.res.ColorStateList;
import android.content.res.Configuration;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.PaintFlagsDrawFilter;
import android.graphics.Path;
import android.graphics.PointF;
import android.graphics.PorterDuff;
import android.graphics.PorterDuffColorFilter;
import android.graphics.Rect;
import android.graphics.RectF;
import android.os.Handler;
import android.os.Looper;
import android.util.SparseArray;
import android.view.MotionEvent;
import android.view.View;
import android.widget.ImageView;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.foxit.sdk.PDFException;
import com.foxit.sdk.PDFViewCtrl;
import com.foxit.sdk.pdf.PSI;
import com.foxit.sdk.pdf.annots.Annot;
import com.foxit.uiextensions.IUIInteractionEventListener;
import com.foxit.uiextensions.IUndoItem;
import com.foxit.uiextensions.Module;
import com.foxit.uiextensions.R;
import com.foxit.uiextensions.ToolHandler;
import com.foxit.uiextensions.UIExtensionsManager;
import com.foxit.uiextensions.annots.AbstractToolHandler;
import com.foxit.uiextensions.annots.common.UIAnnotFrame;
import com.foxit.uiextensions.config.JsonConstants;
import com.foxit.uiextensions.config.uisettings.annotations.annots.PencilConfig;
import com.foxit.uiextensions.controls.propertybar.PropertyBar;
import com.foxit.uiextensions.controls.toolbar.IToolSupply;
import com.foxit.uiextensions.controls.toolbar.ToolConstants;
import com.foxit.uiextensions.controls.toolbar.ToolItemBean;
import com.foxit.uiextensions.controls.toolbar.ToolProperty;
import com.foxit.uiextensions.controls.toolbar.ToolbarItemConfig;
import com.foxit.uiextensions.controls.toolbar.impl.ToolSupplyImpl;
import com.foxit.uiextensions.controls.toolbar.impl.UIColorItem;
import com.foxit.uiextensions.modules.UndoModule;
import com.foxit.uiextensions.theme.ThemeConfig;
import com.foxit.uiextensions.theme.ThemeUtil;
import com.foxit.uiextensions.utils.AppDmUtil;
import com.foxit.uiextensions.utils.AppResource;
import com.foxit.uiextensions.utils.AppUtil;
import com.foxit.uiextensions.utils.Event;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.Callable;

import io.reactivex.Single;
import io.reactivex.disposables.Disposable;
import io.reactivex.schedulers.Schedulers;


public class InkToolHandler extends AbstractToolHandler {
    public static final float IA_MIN_DIST = 0.5f;
    protected static final String PROPERTY_KEY = "INK";

    protected InkAnnotHandler mAnnotHandler;
    protected InkAnnotUtil mUtil;

    private boolean mTouchCaptured = false;
    private int mCapturedPage = -1;

    private final SparseArray<ArrayList<Path>> mPathListArray;
    private final SparseArray<ArrayList<ArrayList<PointF>>> mLineListArray;
    private final SparseArray<ArrayList<ArrayList<PointF>>> mRedoLineListArray;
    private final SparseArray<ArrayList<Path>> mRedoPathListArray;

    private ArrayList<PointF> mLine;
    private ArrayList<PSIData> mCurPSIData;

    private final SparseArray<ArrayList<ArrayList<PSIData>>> mPSIDataListArray;
    private final SparseArray<ArrayList<ArrayList<PSIData>>> mRedoPSIDataListArray;

    private final SparseArray<ArrayList<IUndoItem>> mUndoItemsArray;
    private final SparseArray<ArrayList<IUndoItem>> mRedoItemsArray;

    private Path mPath;
    private final PointF mLastPt = new PointF(0, 0);
    private final Paint mPaint;
    private final PaintFlagsDrawFilter mPaintFilter;
    private final RectF mInvildRect = new RectF();
    int mPencilType;
    private int mUnRedoCount;

    private final PsiBitmap mPsiBmp;
    private PSI mPsi = null;
    private String mNM;

    private Disposable mDisposable;

    private boolean mbHighlighter = false;
    private int mFirstDown = 0;
    private boolean mIsPen = false;
    private boolean mShowPenOnlySwitch = true;
    boolean mIsAddingAnnot = false;
    private long mInkTimeout = -1;

    public InkToolHandler(Context context, PDFViewCtrl pdfViewCtrl, InkAnnotUtil util) {
        super(context, pdfViewCtrl, Module.MODULE_NAME_INK, PROPERTY_KEY);
        mLineListArray = new SparseArray<>();
        mPathListArray = new SparseArray<>();
        mUndoItemsArray = new SparseArray<>();
        mRedoItemsArray = new SparseArray<>();
        mRedoLineListArray = new SparseArray<>();
        mRedoPathListArray = new SparseArray<>();
        mPSIDataListArray = new SparseArray<>();
        mRedoPSIDataListArray = new SparseArray<>();
        mPsiBmp = new PsiBitmap();

        mColor = PropertyBar.PB_COLORS_TOOL_DEFAULT[0];
        mUtil = util;

        mPaint = new Paint();
        mPaint.setStyle(Style.STROKE);
        mPaint.setAntiAlias(true);
        mPaint.setDither(true);
        mPaintFilter = new PaintFlagsDrawFilter(0, Paint.ANTI_ALIAS_FLAG | Paint.FILTER_BITMAP_FLAG);

        mPencilType = InkConstants.PENCIL_WITH_PEN;
        if (mUiExtensionsManager.getConfig().modules.annotations.isLoadPencil) {
            PencilConfig pencilConfig = mUiExtensionsManager.getConfig().uiSettings.annotations.pencil;
            mColor = pencilConfig.color;
            mOpacity = (int) (pencilConfig.opacity * 100);
            mThickness = pencilConfig.thickness;
            mShowPenOnlySwitch = mUiExtensionsManager.getConfig().uiSettings.showPenOnlySwitch;
            setColorChangeListener(new ColorChangeListener() {
                @Override
                public void onColorChange(int color) {
                    if (mCurToolItem == null) return;
                    mCurToolItem.property.color = color;
                    ((UIColorItem) mCurToolItem.toolItem).setAlphaColorBg(color);
                }
            });
        }
    }

    @Override
    public String getType() {
        return ToolHandler.TH_TYPE_INK;
    }

    protected void uninitUiElements() {
        removeToolButton();
    }

    @Override
    public void updateToolButtonStatus() {
        UndoModule undoModule = (UndoModule) mUiExtensionsManager.getModuleByName(Module.MODULE_NAME_UNDO);
        if (undoModule != null)
            undoModule.changeButtonStatus();
    }

    @Override
    public void setColor(final int color) {
        if (mColor == color) return;
        addAnnot(mCapturedPage, true, new Event.Callback() {
            @Override
            public void result(Event event, boolean success) {
                if (mPencilType == InkConstants.PENCIL_WITH_BRUSH) {
                    initPsiCanvas();
                }
            }
        });

        mColor = color;
        if (mPsi != null)
            setPSIColor(color);
        if (mCurToolItem == null) return;
        mCurToolItem.property.color = color;
        if (mCustomPropertyListener != null)
            mCustomPropertyListener.onValueChanged(PropertyBar.PROPERTY_COLOR, color);
    }

    @Override
    public void setOpacity(final int opacity) {
        if (mOpacity == opacity) return;
        addAnnot(mCapturedPage, true, new Event.Callback() {
            @Override
            public void result(Event event, boolean success) {
                if (mPencilType == InkConstants.PENCIL_WITH_BRUSH) {
                    initPsiCanvas();
                }
            }
        });

        mOpacity = opacity;
        if (mPsi != null)
            setPSIOpacity(opacity);
        if (mCurToolItem == null) return;
        mCurToolItem.property.opacity = opacity;
        if (mCustomPropertyListener != null)
            mCustomPropertyListener.onValueChanged(PropertyBar.PROPERTY_OPACITY, opacity);
    }

    @Override
    public void setThickness(final float thickness) {
        if (mThickness == thickness) return;
        addAnnot(mCapturedPage, true, new Event.Callback() {
            @Override
            public void result(Event event, boolean success) {
                if (mPencilType == InkConstants.PENCIL_WITH_BRUSH) {
                    initPsiCanvas();
                }
            }
        });

        mThickness = thickness;
        if (mPsi != null)
            setPSIThickness(thickness);
        if (mCurToolItem == null) return;
        mCurToolItem.property.lineWidth = thickness;
        if (mCustomPropertyListener != null)
            mCustomPropertyListener.onValueChanged(PropertyBar.PROPERTY_LINEWIDTH, thickness);
    }

    @Override
    public void onValueChanged(long property, final int value) {
        super.onValueChanged(property, value);
        if (property == PropertyBar.PROPERTY_PENCIL_TYPE) {
            if (mPencilType == value) return;
            addAnnot(mCapturedPage, true, new Event.Callback() {
                @Override
                public void result(Event event, boolean success) {
                    mPencilType = value;
                    if (mPencilType == InkConstants.PENCIL_WITH_BRUSH)
                        initPsiCanvas();
                    if (mCurToolItem == null) return;
                    mCurToolItem.property.style = value;
                    if (mCustomPropertyListener != null)
                        mCustomPropertyListener.onValueChanged(PropertyBar.PROPERTY_PENCIL_TYPE, mPencilType);
                }
            });
        }
    }

    @Override
    public void onActivate() {
        mInkTimeout = mUiExtensionsManager.getInkDrawingTimeout();
        mIsPen = mUiExtensionsManager.getInkDrawToolType() == InkDrawToolType.STYLUS;
        mFirstDown = 0;
        mCapturedPage = -1;

        mLineListArray.clear();
        mPathListArray.clear();
        mPSIDataListArray.clear();
        resetPropertyBar();
        releasePsiCanvas();
        if (mPencilType == InkConstants.PENCIL_WITH_BRUSH) {
            initPsiCanvas();
        }

        final UndoModule undoModule = (UndoModule) mUiExtensionsManager.getModuleByName(Module.MODULE_NAME_UNDO);
        if (undoModule != null) {
            undoModule.setUndoItemCallback(new UndoModule.IUndoItemCallback() {
                @Override
                public boolean undo() {
                    if (mCapturedPage == -1) return false;

                    ArrayList<ArrayList<PointF>> lineList = mLineListArray.get(mCapturedPage);
                    ArrayList<IUndoItem> undoItems = mUndoItemsArray.get(mCapturedPage);
                    if (lineList == null || undoItems == null) return false;

                    if (mUnRedoCount > 0) {
                        if (mUiExtensionsManager.getDocumentManager().canUndo()) {
                            mUiExtensionsManager.getDocumentManager().undo();
                            mUnRedoCount--;
                            return true;
                        }
                    } else if (lineList.size() > 0) {
                        // MOBRD-6394
                        // Exception java.lang.IndexOutOfBoundsException: Index: 11, Size: 11
                        //  at java.util.ArrayList.remove (ArrayList.java:503)
                        //  at com.foxit.uiextensions.annots.ink.InkToolHandler$6.undo (InkToolHandler.java:289)
                        if (lineList.size() - 1 >= undoItems.size()) {
                            lineList.remove(lineList.size() - 1);
                            return true;
                        }

                        IUndoItem undoItem = undoItems.remove(lineList.size() - 1);
                        ArrayList<IUndoItem> redoItems = mRedoItemsArray.get(mCapturedPage);
                        if (redoItems == null) {
                            redoItems = new ArrayList<>();
                            mRedoItemsArray.put(mCapturedPage, redoItems);
                        }
                        redoItems.add(undoItem);

                        ArrayList<ArrayList<PointF>> redoLineList = mRedoLineListArray.get(mCapturedPage);
                        if (redoLineList == null) {
                            redoLineList = new ArrayList<>();
                            mRedoLineListArray.put(mCapturedPage, redoLineList);
                        }
                        redoLineList.add(lineList.remove(lineList.size() - 1));

                        if (mPencilType == InkConstants.PENCIL_WITH_PEN) {
                            ArrayList<Path> pathList = mPathListArray.get(mCapturedPage);
                            if (pathList != null) {
                                ArrayList<Path> redoPathList = mRedoPathListArray.get(mCapturedPage);
                                if (redoPathList == null) {
                                    redoPathList = new ArrayList<>();
                                    mRedoPathListArray.put(mCapturedPage, redoPathList);
                                }
                                redoPathList.add(pathList.remove(pathList.size() - 1));
                            }
                            undoModule.changeButtonStatus();
                            if (mPdfViewCtrl.isPageVisible(mCapturedPage)) {
                                RectF rectF = new RectF(mInvildRect);
                                mPdfViewCtrl.convertPageViewRectToDisplayViewRect(rectF, rectF, mCapturedPage);
                                mPdfViewCtrl.invalidate(AppDmUtil.rectFToRect(rectF));
                            }
                            return true;
                        } else {
                            ArrayList<ArrayList<PSIData>> psiDataList = mPSIDataListArray.get(mCapturedPage);
                            if (psiDataList != null) {
                                ArrayList<ArrayList<PSIData>> redoPsiDataList = mRedoPSIDataListArray.get(mCapturedPage);
                                if (redoPsiDataList == null) {
                                    redoPsiDataList = new ArrayList<>();
                                    mRedoPSIDataListArray.put(mCapturedPage, redoPsiDataList);
                                }
                                redoPsiDataList.add(psiDataList.remove(psiDataList.size() - 1));
                            }
                            undoModule.changeButtonStatus();

                            RectF refreshRect = new RectF();
                            refreshRect.set(mPSIRectF);
                            mPSIRectF.setEmpty();
                            releasePsiCanvas();
                            initPsiCanvas();

                            try {
                                if (psiDataList != null) {
                                    for (ArrayList<PSIData> psiDataArrayList : psiDataList) {
                                        for (PSIData psiData : psiDataArrayList) {
                                            PointF point = psiData.pointF;
                                            if (mPSIRectF.isEmpty()) {
                                                mPSIRectF.left = point.x;
                                                mPSIRectF.top = point.y;
                                            }
                                            mPsi.addPoint(AppUtil.toFxPointF(point), psiData.path_type, psiData.pressure);
                                            mPSIRectF.union(point.x, point.y);
                                        }
                                    }
                                }
                                mPageWidth = mPdfViewCtrl.getPageViewWidth(mCapturedPage);
                                mPageHeight = mPdfViewCtrl.getPageViewHeight(mCapturedPage);
                                mPsiBmp.mPvSize = new PointF(mPageWidth, mPageHeight);

                                mPsiBmp.mBmp = mPsi.getBitmap();
                                mPsiBmp.mPage = mCapturedPage;

                                mPsiBmp.mLeftTop = new PointF(0, 0);
                                mPdfViewCtrl.convertPageViewPtToDisplayViewPt(mPsiBmp.mLeftTop, mPsiBmp.mLeftTop, mCapturedPage);
                            } catch (PDFException e) {
                                e.printStackTrace();
                            }

                            refreshRect.union(mPSIRectF);
                            if (mPdfViewCtrl.isPageVisible(mCapturedPage)) {
                                mPdfViewCtrl.invalidate(AppDmUtil.rectFToRect(refreshRect));
                            }
                            return true;
                        }
                    } else {
                        if (mUiExtensionsManager.getDocumentManager().canUndo()) {
                            mUiExtensionsManager.getDocumentManager().undo();
                            mUnRedoCount--;
                            return true;
                        }
                    }
                    return false;
                }

                @Override
                public boolean canUndo() {
                    return mUiExtensionsManager.getDocumentManager().canUndo()
                            || (mLineListArray.get(mCapturedPage) != null && mLineListArray.get(mCapturedPage).size() > 0);
                }

                @Override
                public boolean redo() {
                    if (mCapturedPage == -1) return false;

                    ArrayList<ArrayList<PointF>> redoLineList = mRedoLineListArray.get(mCapturedPage);
                    ArrayList<IUndoItem> redoItems = mRedoItemsArray.get(mCapturedPage);
                    if (redoLineList == null || redoItems == null) return false;

                    if (mUnRedoCount < 0) {
                        if (mUiExtensionsManager.getDocumentManager().canRedo()) {
                            mUiExtensionsManager.getDocumentManager().redo();
                            mUnRedoCount++;
                            return true;
                        }
                    } else if (redoLineList.size() > 0) {
                        ArrayList<IUndoItem> undoItems = mUndoItemsArray.get(mCapturedPage);
                        if (undoItems == null) {
                            undoItems = new ArrayList<>();
                            mUndoItemsArray.put(mCapturedPage, undoItems);
                        }
                        undoItems.add(redoItems.remove(redoItems.size() - 1));

                        ArrayList<ArrayList<PointF>> lineList = mLineListArray.get(mCapturedPage);
                        if (lineList == null) {
                            lineList = new ArrayList<>();
                            mLineListArray.put(mCapturedPage, lineList);
                        }
                        lineList.add(redoLineList.remove(redoLineList.size() - 1));

                        if (mPencilType == InkConstants.PENCIL_WITH_PEN) {
                            ArrayList<Path> redoPathList = mRedoPathListArray.get(mCapturedPage);
                            if (redoPathList != null) {
                                ArrayList<Path> pathList = mPathListArray.get(mCapturedPage);
                                if (pathList == null) {
                                    pathList = new ArrayList<>();
                                    mPathListArray.put(mCapturedPage, pathList);
                                }
                                pathList.add(redoPathList.remove(redoPathList.size() - 1));
                            }
                            undoModule.changeButtonStatus();
                            if (mPdfViewCtrl.isPageVisible(mCapturedPage)) {
                                RectF rectF = new RectF(mInvildRect);
                                mPdfViewCtrl.convertPageViewRectToDisplayViewRect(rectF, rectF, mCapturedPage);
                                mPdfViewCtrl.invalidate(AppDmUtil.rectFToRect(rectF));
                            }
                            return true;
                        } else {
                            ArrayList<ArrayList<PSIData>> redoPsiList = mRedoPSIDataListArray.get(mCapturedPage);
                            ArrayList<ArrayList<PSIData>> psiList = null;
                            if (redoPsiList != null) {
                                psiList = mPSIDataListArray.get(mCapturedPage);
                                if (psiList == null) {
                                    psiList = new ArrayList<>();
                                    mPSIDataListArray.put(mCapturedPage, psiList);
                                }
                                psiList.add(redoPsiList.remove(redoPsiList.size() - 1));
                            }
                            undoModule.changeButtonStatus();

                            RectF refreshRect = new RectF();
                            refreshRect.set(mPSIRectF);
                            mPSIRectF.setEmpty();
                            releasePsiCanvas();
                            initPsiCanvas();
                            try {
                                if (psiList != null) {
                                    for (ArrayList<PSIData> psiDataArrayList : psiList) {
                                        for (PSIData psiData : psiDataArrayList) {
                                            PointF point = psiData.pointF;
                                            if (mPSIRectF.isEmpty()) {
                                                mPSIRectF.left = point.x;
                                                mPSIRectF.top = point.y;
                                            }
                                            mPsi.addPoint(AppUtil.toFxPointF(point), psiData.path_type, psiData.pressure);
                                            mPSIRectF.union(point.x, point.y);
                                        }
                                    }
                                }
                                mPageWidth = mPdfViewCtrl.getPageViewWidth(mCapturedPage);
                                mPageHeight = mPdfViewCtrl.getPageViewHeight(mCapturedPage);
                                mPsiBmp.mPvSize = new PointF(mPageWidth, mPageHeight);

                                mPsiBmp.mBmp = mPsi.getBitmap();
                                mPsiBmp.mPage = mCapturedPage;

                                mPsiBmp.mLeftTop = new PointF(0, 0);
                                mPdfViewCtrl.convertPageViewPtToDisplayViewPt(mPsiBmp.mLeftTop, mPsiBmp.mLeftTop, mCapturedPage);
                            } catch (PDFException e) {
                                e.printStackTrace();
                            }
                            refreshRect.union(mPSIRectF);
                            if (mPdfViewCtrl.isPageVisible(mCapturedPage)) {
                                mPdfViewCtrl.invalidate(AppDmUtil.rectFToRect(refreshRect));
                            }
                            return true;
                        }
                    } else {
                        if (mUiExtensionsManager.getDocumentManager().canRedo()) {
                            mUiExtensionsManager.getDocumentManager().redo();
                            mUnRedoCount++;
                            return true;
                        }
                    }
                    return false;
                }

                @Override
                public boolean canRedo() {
                    return mUiExtensionsManager.getDocumentManager().canRedo()
                            || (mRedoLineListArray.get(mCapturedPage) != null && mRedoLineListArray.get(mCapturedPage).size() > 0);
                }
            });

            undoModule.changeButtonStatus();
        }
    }

    @Override
    protected void resetPropertyBar() {
        int[] colors = new int[PropertyBar.PB_COLORS_TOOL_DEFAULT.length];
        System.arraycopy(PropertyBar.PB_COLORS_TOOL_DEFAULT, 0, colors, 0, colors.length);
        mPropertyBar.setColors(colors);
        mPropertyBar.setProperty(PropertyBar.PROPERTY_COLOR, mColor);
        mPropertyBar.setProperty(PropertyBar.PROPERTY_OPACITY, mOpacity);
        mPropertyBar.setProperty(PropertyBar.PROPERTY_LINEWIDTH, mThickness);
        if (!mbHighlighter) {
            mPropertyBar.setProperty(PropertyBar.PROPERTY_PENCIL_TYPE, mPencilType);
            mPropertyBar.setProperty(PropertyBar.PROPERTY_MAX_LINEWIDTH, 21);
        } else {
            mPropertyBar.setProperty(PropertyBar.PROPERTY_MAX_LINEWIDTH, 45);
        }

        mPropertyBar.clearPropertyTitle();
        mPropertyBar.reset(getSupportedProperties());
        if (mShowPenOnlySwitch)
            mPropertyBar.addCustomItem(PropertyBar.PROPERTY_PEN_ONLY, getPenOnlyView(), 0, -1);
        mPropertyBar.setPropertyChangeListener(this);
    }

    @Override
    protected void onCreateValueChanged(long property, Object value) {
        if (property == PropertyBar.PROPERTY_PENCIL_TYPE) {
            if (mPencilType == (int) value) return;
            mPencilType = (int) value;
        }
    }

    private View mPenOnlyView;
    private RelativeLayout mPenOnlySwitchRela;
    private ImageView mPenOnlySwitchIv;
    private TextView mPenOnlyTitleTv;

    private View getPenOnlyView() {
        mPenOnlyView = View.inflate(mContext, R.layout.pb_ink_draw_type, null);
        mPenOnlyTitleTv = mPenOnlyView.findViewById(R.id.ink_pen_only_switch_tv);
        mPenOnlySwitchRela = mPenOnlyView.findViewById(R.id.ink_pen_only_switch_rl);
        mPenOnlySwitchIv = mPenOnlyView.findViewById(R.id.ink_pen_only_switch_iv);
        ThemeUtil.setBackgroundTintList(mPenOnlySwitchRela, getSelectedButtonColorStateList());
        setSelectedButtonState(mPenOnlySwitchRela, mPenOnlySwitchIv, mUiExtensionsManager.getInkDrawToolType() == InkDrawToolType.STYLUS);

        mPenOnlySwitchRela.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                mFirstDown = 0;
                if (mPenOnlySwitchRela.isSelected()) {
                    mIsPen = false;
                    setSelectedButtonState(mPenOnlySwitchRela, mPenOnlySwitchIv, false);
                    if (mCurToolItem != null) {
                        mUiExtensionsManager.setInkDrawToolType(InkDrawToolType.STYLUS_OR_FINGER);
                    }
                } else {
                    mIsPen = true;
                    setSelectedButtonState(mPenOnlySwitchRela, mPenOnlySwitchIv, true);
                    if (mCurToolItem != null) {
                        mUiExtensionsManager.setInkDrawToolType(InkDrawToolType.STYLUS);
                    }
                }
            }
        });
        return mPenOnlyView;
    }

    private void setSelectedButtonState(RelativeLayout checkLayout, ImageView checkView, boolean isSelected) {
        checkLayout.setSelected(isSelected);
        RelativeLayout.LayoutParams params = (RelativeLayout.LayoutParams) checkView.getLayoutParams();
        if (params == null) return;
        params.removeRule(isSelected ? RelativeLayout.ALIGN_PARENT_LEFT : RelativeLayout.ALIGN_PARENT_RIGHT);
        params.addRule(isSelected ? RelativeLayout.ALIGN_PARENT_RIGHT : RelativeLayout.ALIGN_PARENT_LEFT, RelativeLayout.TRUE);
        checkView.setLayoutParams(params);
    }

    void updateTheme() {
        if (mPenOnlyView != null) {
            mPenOnlyTitleTv.setTextColor(AppResource.getColor(mContext, R.color.t2));
            ThemeUtil.setBackgroundTintList(mPenOnlySwitchRela, getSelectedButtonColorStateList());
            setSelectedButtonState(mPenOnlySwitchRela, mPenOnlySwitchIv, mUiExtensionsManager.getInkDrawToolType() == InkDrawToolType.STYLUS);
        }
    }

    private ColorStateList getSelectedButtonColorStateList() {
        int disabled = AppResource.getColor(mContext, R.color.p1);
        int selected = ThemeConfig.getInstance(mContext).getPrimaryColor();
        int normal = AppResource.getColor(mContext, R.color.p1);
        return AppResource.createColorStateList(selected, disabled, normal);
    }

    @Override
    public void onDeactivate() {
        if (mDisposable != null) {
            if (!mDisposable.isDisposed())
                mDisposable.dispose();
            mDisposable = null;
        }

        mIsPen = false;
        mFirstDown = 0;
        UndoModule undoModule = (UndoModule) mUiExtensionsManager.getModuleByName(Module.MODULE_NAME_UNDO);
        if (undoModule != null)
            undoModule.setUndoItemCallback(null);
        if (mTouchCaptured) {
            mTouchCaptured = false;

            if (mLine != null) {
                _addLine(mLine);
                mLine = null;
            }
            mLastPt.set(0, 0);
        }

        addAnnot(mCapturedPage, true, new Event.Callback() {
            @Override
            public void result(Event event, boolean success) {
                if (mUiExtensionsManager.getCurrentToolHandler() == InkToolHandler.this) {
                    if (mPencilType == InkConstants.PENCIL_WITH_BRUSH)
                        initPsiCanvas();
                } else {
                    mCapturedPage = -1;
                }
            }
        });
    }

    float mbx, mby, mcx, mcy, mex, mey;
    PointF tv_pt = new PointF();
    RectF tv_invalid = new RectF();
    private final RectF mPSIRectF = new RectF();
    private int mPageWidth;
    private int mPageHeight;

    @Override
    public boolean onTouchEvent(int pageIndex, final MotionEvent e) {
        boolean handled = true;
        int action = e.getActionMasked();

        switch (action) {
            case MotionEvent.ACTION_DOWN:
                if (mFirstDown < 3) {
                    mFirstDown++;
                }
                if (mFirstDown == 1 && e.getToolType(0) == MotionEvent.TOOL_TYPE_STYLUS) {
                    mIsPen = true;
                }
                if (e.getToolType(0) == MotionEvent.TOOL_TYPE_STYLUS && !mIsPen) {
                    handled = mUiExtensionsManager.defaultTouchEvent(pageIndex, e);
                    break;
                }
                if (e.getToolType(0) != MotionEvent.TOOL_TYPE_STYLUS && mIsPen) {
                    handled = mUiExtensionsManager.defaultTouchEvent(pageIndex, e);
                    break;
                }
                onInkToolTouch(pageIndex, e);
                handled = true;
                break;
            case MotionEvent.ACTION_POINTER_DOWN:
                isPointerDown(pageIndex, e);
                MotionEvent downEvent = MotionEvent.obtain(e);
                downEvent.setAction(MotionEvent.ACTION_DOWN);
                handled = mPdfViewCtrl.defaultTouchEvent(downEvent);
                downEvent.recycle();
                break;
            case MotionEvent.ACTION_MOVE:
                if (!mTouchCaptured && e.getPointerCount() > 1) {
                    handled = mPdfViewCtrl.defaultTouchEvent(e);
                } else {
                    if (e.getToolType(0) != MotionEvent.TOOL_TYPE_STYLUS && mIsPen) {
                        handled = mUiExtensionsManager.defaultTouchEvent(pageIndex, e);
                    } else {
                        handled = onInkToolTouch(pageIndex, e);
                    }
                }
                break;
            case MotionEvent.ACTION_CANCEL:
            case MotionEvent.ACTION_UP:
                if (!mTouchCaptured) {
                    handled = mPdfViewCtrl.defaultTouchEvent(e);
                } else {
                    if (e.getToolType(0) != MotionEvent.TOOL_TYPE_STYLUS && mIsPen) {
                        handled = mUiExtensionsManager.defaultTouchEvent(pageIndex, e);
                    } else {
                        handled = onInkToolTouch(pageIndex, e);
                    }
                }

                break;
            default:
                break;
        }
        return handled;
    }

    private void isPointerDown(int pageIndex, MotionEvent e) {
        addAnnot(pageIndex, true, new Event.Callback() {
            @Override
            public void result(Event event, boolean success) {
                if (mPencilType == InkConstants.PENCIL_WITH_BRUSH)
                    initPsiCanvas();
            }
        });
        mLine = null;
        mTouchCaptured = false;
        mLastPt.set(0, 0);
        mPath = null;
//                mCapturedPage = -1;
    }

    private final ArrayList<PointF> mPsiPoints = new ArrayList<>();
    private final ArrayList<Float> mPsiPresses = new ArrayList<>();

    private boolean onInkToolTouch(final int pageIndex, MotionEvent e) {
        PointF point = new PointF(e.getX(), e.getY());
        mPdfViewCtrl.convertDisplayViewPtToPageViewPt(point, point, pageIndex);
        float thickness = UIAnnotFrame.getPageViewThickness(mPdfViewCtrl, pageIndex, mThickness);
        float pressure = calibrationPressure(e.getPressure());
        int action = e.getAction();
        switch (action) {
            case MotionEvent.ACTION_DOWN:
                if (!mTouchCaptured) {
                    this.cancelInkTimeOut();

                    if (mCapturedPage == -1) {
                        mTouchCaptured = true;
                        mCapturedPage = pageIndex;
                    } else if (pageIndex == mCapturedPage) {
                        mTouchCaptured = true;
                    } else {
                        // support add ink to multiple pages
                        releasePsiCanvas();
                        initPsiCanvas();

                        addAnnot(mCapturedPage, true, false, null);
                        mTouchCaptured = true;
                        mCapturedPage = pageIndex;
                    }

                    if (mPencilType == InkConstants.PENCIL_WITH_PEN) {
                        mPath = new Path();
                        mPath.moveTo(point.x, point.y);
                        mbx = point.x;
                        mby = point.y;
                        mcx = point.x;
                        mcy = point.y;
                    } else {
                        mPageWidth = mPdfViewCtrl.getPageViewWidth(pageIndex);
                        mPageHeight = mPdfViewCtrl.getPageViewHeight(pageIndex);
                        if (mPsiBmp.mLeftTop == null || mPsiBmp.mPage != mCapturedPage) {
                            mPsiBmp.mPage = pageIndex;
                            mPsiBmp.mLeftTop = new PointF(0, 0);
                            mPdfViewCtrl.convertPageViewPtToDisplayViewPt(mPsiBmp.mLeftTop, mPsiBmp.mLeftTop, pageIndex);
                            mPsiBmp.mPvSize = new PointF(mPageWidth, mPageHeight);
                        }

                        PointF psiPointF = new PointF();
                        mPdfViewCtrl.convertPageViewPtToDisplayViewPt(point, psiPointF, pageIndex);

                        mCurPSIData = new ArrayList<>();
                        PSIData psiData = new PSIData();
                        psiData.pointF = new PointF(psiPointF.x, psiPointF.y);
                        psiData.path_type = com.foxit.sdk.common.Path.e_TypeMoveTo;
                        psiData.pressure = pressure;
                        mCurPSIData.add(psiData);

                        PointF pt = new PointF(psiPointF.x, psiPointF.y);
                        if (mPSIRectF.isEmpty()) {
                            mPSIRectF.left = psiPointF.x;
                            mPSIRectF.top = psiPointF.y;
                        }
                        ArrayList<PointF> points = new ArrayList<>();
                        points.add(pt);
                        ArrayList<Float> presses = new ArrayList<>();
                        presses.add(pressure);

                        addPsiPoint(points, presses, com.foxit.sdk.common.Path.e_TypeMoveTo);
                    }
                    mLine = new ArrayList<>();
                    mLine.add(new PointF(point.x, point.y));
                    mLastPt.set(point.x, point.y);
                }
                return true;
            case MotionEvent.ACTION_MOVE:
            case MotionEvent.ACTION_UP:
            case MotionEvent.ACTION_CANCEL:

                if (mTouchCaptured) {
                    tv_pt.set(point);
                    InkAnnotUtil.correctPvPoint(mPdfViewCtrl, pageIndex, tv_pt);
                    float dx = Math.abs(tv_pt.x - mcx);
                    float dy = Math.abs(tv_pt.y - mcy);
                    if (mCapturedPage == pageIndex) {
                        PencilConfig pencilConfig = mUiExtensionsManager.getConfig().uiSettings.annotations.pencil;
                        boolean dist = (dx >= IA_MIN_DIST || dy >= IA_MIN_DIST);
                        if (mCapturedPage == pageIndex && dist) {
                            if (mPencilType == InkConstants.PENCIL_WITH_PEN) {                            // history pmCurPSIDataoints
                                tv_invalid.set(tv_pt.x, tv_pt.y, tv_pt.x, tv_pt.y);
                                if (pencilConfig.addHistoricalPoints) {
                                    int historySize = e.getHistorySize();
                                    for (int i = 0; i < historySize; i++) {
                                        tv_pt.set(e.getHistoricalX(i), e.getHistoricalY(i));
                                        mPdfViewCtrl.convertDisplayViewPtToPageViewPt(tv_pt, tv_pt, pageIndex);
                                        InkAnnotUtil.correctPvPoint(mPdfViewCtrl, pageIndex, tv_pt);
                                        if (tv_pt.x - mLastPt.x >= IA_MIN_DIST || tv_pt.y - mLastPt.y >= IA_MIN_DIST) {
                                            mex = (mcx + tv_pt.x) / 2;
                                            mey = (mcy + tv_pt.y) / 2;
                                            mLine.add(new PointF(tv_pt.x, tv_pt.y));
                                            mPath.quadTo(mcx, mcy, mex, mey);
                                            mLastPt.set(tv_pt);
                                            tv_invalid.union(mbx, mby);
                                            tv_invalid.union(mcx, mcy);
                                            tv_invalid.union(mex, mey);
                                            mbx = mex;
                                            mby = mey;
                                            mcx = tv_pt.x;
                                            mcy = tv_pt.y;
                                        }
                                    }
                                }
                                // current point
                                tv_pt.set(point);
                                InkAnnotUtil.correctPvPoint(mPdfViewCtrl, pageIndex, tv_pt);
                                mex = (mcx + tv_pt.x) / 2;
                                mey = (mcy + tv_pt.y) / 2;
                                mLine.add(new PointF(tv_pt.x, tv_pt.y));
                                mPath.quadTo(mcx, mcy, mex, mey);
                                mLastPt.set(tv_pt.x, tv_pt.y);
                                tv_invalid.union(mbx, mby);
                                tv_invalid.union(mcx, mcy);
                                tv_invalid.union(mex, mey);
                                mbx = mex;
                                mby = mey;
                                mcx = tv_pt.x;
                                mcy = tv_pt.y;
                                tv_invalid.inset(-thickness, -thickness);
                                mPdfViewCtrl.convertPageViewRectToDisplayViewRect(tv_invalid, tv_invalid, pageIndex);
                                mPdfViewCtrl.invalidate(AppDmUtil.rectFToRect(tv_invalid));
                            } else {
                                PointF psiPointF = new PointF();
                                PSIData psiData;
                                mPsiPoints.clear();
                                mPsiPresses.clear();
                                if (pencilConfig.addHistoricalPoints) {
                                    int historySize = e.getHistorySize();
                                    for (int i = 0; i < historySize; i++) {
                                        tv_pt.set(e.getHistoricalX(i), e.getHistoricalY(i));
                                        mPdfViewCtrl.convertDisplayViewPtToPageViewPt(tv_pt, tv_pt, pageIndex);
                                        InkAnnotUtil.correctPvPoint(mPdfViewCtrl, pageIndex, tv_pt);
                                        if (tv_pt.x - mLastPt.x >= IA_MIN_DIST || tv_pt.y - mLastPt.y >= IA_MIN_DIST) {
                                            psiPointF.set(e.getHistoricalX(i), e.getHistoricalY(i));
                                            mLine.add(new PointF(tv_pt.x, tv_pt.y));
                                            mLastPt.set(mLine.get(mLine.size() - 1));
                                            psiData = new PSIData();
                                            psiData.pointF = new PointF(psiPointF.x, psiPointF.y);
                                            float historicalPressure = calibrationPressure(e.getHistoricalPressure(i));
                                            psiData.pressure = historicalPressure;
                                            psiData.path_type = com.foxit.sdk.common.Path.e_TypeLineTo;
                                            mCurPSIData.add(psiData);
                                            mPsiPoints.add(new PointF(psiPointF.x, psiPointF.y));
                                            mPsiPresses.add(historicalPressure);
                                        }
                                    }
                                }
                                addPsiPoint(mPsiPoints, mPsiPresses, com.foxit.sdk.common.Path.e_TypeLineTo);

                                // current point
                                tv_pt.set(point);
                                InkAnnotUtil.correctPvPoint(mPdfViewCtrl, pageIndex, tv_pt);

                                if (!tv_pt.equals(mLastPt)) {
                                    psiPointF = new PointF();
                                    mPdfViewCtrl.convertPageViewPtToDisplayViewPt(tv_pt, psiPointF, pageIndex);

                                    psiData = new PSIData();
                                    psiData.pointF = new PointF(psiPointF.x, psiPointF.y);
                                    psiData.pressure = pressure;
                                    psiData.path_type = com.foxit.sdk.common.Path.e_TypeLineTo;
                                    mCurPSIData.add(psiData);

                                    mLine.add(new PointF(tv_pt.x, tv_pt.y));
                                    mLastPt.set(tv_pt.x, tv_pt.y);
                                    PointF pt = new PointF(psiPointF.x, psiPointF.y);

                                    mPsiPoints.clear();
                                    mPsiPresses.clear();
                                    mPsiPoints.add(pt);
                                    mPsiPresses.add(pressure);
                                    addPsiPoint(mPsiPoints, mPsiPresses, com.foxit.sdk.common.Path.e_TypeLineTo);
                                }
                            }
                        }
                    }
                    if (action == MotionEvent.ACTION_UP || action == MotionEvent.ACTION_CANCEL) {
                        tv_pt.set(point);
                        InkAnnotUtil.correctPvPoint(mPdfViewCtrl, pageIndex, tv_pt);
                        if (mPencilType == InkConstants.PENCIL_WITH_PEN) {
                            if (mLine.size() == 1) {
                                if (tv_pt.equals(mLine.get(0))) {
                                    tv_pt.x += 0.1;
                                    tv_pt.y += 0.1;
                                }
                                mex = (mcx + tv_pt.x) / 2;
                                mey = (mcy + tv_pt.y) / 2;
                                mLine.add(new PointF(tv_pt.x, tv_pt.y));
                                mPath.quadTo(mcx, mcy, mex, mey);
                                mLastPt.set(tv_pt.x, tv_pt.y);
                            }
                            mPath.lineTo(mLastPt.x, mLastPt.y);

                            ArrayList<Path> pathList = mPathListArray.get(mCapturedPage);
                            if (pathList == null) {
                                pathList = new ArrayList<>();
                                mPathListArray.put(mCapturedPage, pathList);
                            }
                            pathList.add(mPath);
                            mPath = null;
                            tv_invalid.set(mbx, mby, mbx, mby);
                            tv_invalid.union(mcx, mcy);
                            tv_invalid.union(mex, mey);
                            tv_invalid.union(mLastPt.x, mLastPt.y);
                            tv_invalid.inset(-thickness, -thickness);
                            mInvildRect.union(new RectF(tv_invalid));
                            mPdfViewCtrl.convertPageViewRectToDisplayViewRect(tv_invalid, tv_invalid, pageIndex);
                            mPdfViewCtrl.invalidate(AppDmUtil.rectFToRect(tv_invalid));
                        } else {
                            if (tv_pt.equals(mLastPt)) {
                                PointF offsetPoint = new PointF(point.x, point.y);
                                float offset = InkAnnotUtil.widthOnPageView(mPdfViewCtrl, pageIndex, 1.5f);
                                offsetPoint.offset(offset, 0);
                                mLine.add(new PointF(offsetPoint.x, offsetPoint.y));

                                PointF psiPointF = new PointF();
                                mPdfViewCtrl.convertPageViewPtToDisplayViewPt(offsetPoint, psiPointF, pageIndex);

                                PSIData psiData = new PSIData();
                                psiData.pointF = new PointF(psiPointF.x, psiPointF.y);
                                psiData.pressure = pressure;
                                psiData.path_type = com.foxit.sdk.common.Path.e_TypeLineTo;
                                mCurPSIData.add(psiData);

                                mPsiPoints.clear();
                                mPsiPresses.clear();
                                mPsiPoints.add(psiPointF);
                                mPsiPresses.add(pressure);
                                addPsiPoint(mPsiPoints, mPsiPresses, com.foxit.sdk.common.Path.e_TypeLineTo);
                            }
                            PointF psiPointF = new PointF();
                            mPdfViewCtrl.convertPageViewPtToDisplayViewPt(tv_pt, psiPointF, pageIndex);
                            PointF pt = new PointF(psiPointF.x, psiPointF.y);
                            mLine.add(new PointF(tv_pt.x, tv_pt.y));

                            mPsiPresses.clear();
                            mPsiPoints.clear();
                            mPsiPoints.add(pt);
                            mPsiPresses.add(pressure);
                            addPsiPoint(mPsiPoints, mPsiPresses, com.foxit.sdk.common.Path.e_TypeLineToCloseFigure);

                            PSIData psiData = new PSIData();
                            psiData.pointF = new PointF(psiPointF.x, psiPointF.y);
                            psiData.pressure = pressure;
                            psiData.path_type = com.foxit.sdk.common.Path.e_TypeLineToCloseFigure;
                            mCurPSIData.add(psiData);
                            ArrayList<ArrayList<PSIData>> psiList = mPSIDataListArray.get(mCapturedPage);
                            if (psiList == null) {
                                psiList = new ArrayList<>();
                                mPSIDataListArray.put(mCapturedPage, psiList);
                            }
                            psiList.add(mCurPSIData);
                            mCurPSIData = null;
                        }
                        _addLine(mLine);
                        mLine = null;
                        mTouchCaptured = false;
                        mLastPt.set(0, 0);

                        addUndoItem(pageIndex, new Event.Callback() {
                            @Override
                            public void result(Event event, boolean success) {
                                if (mInkTimeout > 0) {
                                    handleInkTimeOut();
                                }
                            }
                        });
                    }
                    return true;
                }
                break;
            default:
                return true;
        }
        return true;
    }

    private void addPsiPoint(final List<PointF> points, final List<Float> pressures, final int flag) {
        try {
            if (mPsi == null || mPsi.isEmpty() || points.size() == 0) return;
            for (int i = 0; i < points.size(); i++) {
                PointF point = points.get(i);
                float pressure = 1.0f;
                if (pressures != null && pressures.get(i) < 1.0f) {
                    pressure = pressures.get(i);
                }
                mPsi.addPoint(AppUtil.toFxPointF(point), flag, pressure);
                mPSIRectF.union(point.x, point.y);
            }

            mPsiBmp.mBmp = mPsi.getBitmap();
            if (mPdfViewCtrl.isPageVisible(mCapturedPage)) {
                RectF rectF = new RectF(mPSIRectF);
                mPdfViewCtrl.invalidate(AppDmUtil.rectFToRect(rectF));
            }
        } catch (PDFException e) {
            e.printStackTrace();
        }
    }

    void _addLine(ArrayList<PointF> line) {
        ArrayList<ArrayList<PointF>> lineList = mLineListArray.get(mCapturedPage);
        if (lineList == null) {
            lineList = new ArrayList<>();
            mLineListArray.put(mCapturedPage, lineList);
        }
        lineList.add(line);

        clearUndoMembers(mCapturedPage);
        updateToolButtonStatus();
    }

    private void clearUndoMembers(int pageIndex) {
        mUnRedoCount = 0;
        mRedoLineListArray.remove(pageIndex);
        mRedoPathListArray.remove(pageIndex);
        mRedoItemsArray.remove(pageIndex);
        mRedoPSIDataListArray.remove(pageIndex);
    }

    @Override
    public boolean onLongPress(int pageIndex, MotionEvent motionEvent) {
        if (mUiExtensionsManager.getDocumentManager().getCurrentAnnot() != null) {
            return mUiExtensionsManager.defaultSingleTapConfirmed(pageIndex, motionEvent);
        }

        mPdfViewCtrl.capturePageViewOnTouch(motionEvent);
        return onTouchEvent(pageIndex, motionEvent);
    }

    @Override
    public boolean onSingleTapConfirmed(int pageIndex, MotionEvent motionEvent) {
        return mUiExtensionsManager.getCurrentToolHandler() == this;
    }

    private final Rect tv_srcrect = new Rect();
    private final Rect tv_dstrect = new Rect();

    @Override
    public void onDraw(int pageIndex, Canvas canvas) {
        if (mCapturedPage == pageIndex) {
            // draw current creating annotation
            setPaintProperty(mPdfViewCtrl, pageIndex, mPaint);
            canvas.setDrawFilter(mPaintFilter);

            if (mPencilType == InkConstants.PENCIL_WITH_PEN) {
                mPaint.setColorFilter(null);

                ArrayList<Path> pathList = mPathListArray.get(pageIndex);
                if (pathList != null) {
                    int count = pathList.size();
                    for (int i = 0; i < count; i++) {
                        canvas.drawPath(pathList.get(i), mPaint);
                    }
                }

                if (mPath != null) {
                    canvas.drawPath(mPath, mPaint);
                }
            } else {
                if (mPsiBmp.mBmp != null && mPsiBmp.mPage == pageIndex) {
                    mPaint.setColorFilter(new PorterDuffColorFilter(mColor, PorterDuff.Mode.SRC_IN));
                    if (mPsiBmp.mPvSize.x == mPageWidth && mPsiBmp.mPvSize.y == mPageHeight) {
                        canvas.getClipBounds(tv_dstrect);
                        tv_srcrect.set(tv_dstrect);
                        tv_srcrect.offset((int) mPsiBmp.mLeftTop.x, (int) mPsiBmp.mLeftTop.y);
                        tv_srcrect.intersect(0, 0, mPsiBmp.mBmp.getWidth(), mPsiBmp.mBmp.getHeight());
                        tv_dstrect.set(tv_srcrect);
                        tv_dstrect.offset((int) -mPsiBmp.mLeftTop.x, (int) -mPsiBmp.mLeftTop.y);
                        canvas.drawBitmap(mPsiBmp.mBmp, tv_srcrect, tv_dstrect, mPaint);
                    } else {
                        float scale = (float) mPsiBmp.mPvSize.x / mPageWidth;
                        canvas.getClipBounds(tv_dstrect);
                        tv_srcrect.set(tv_dstrect);
                        tv_srcrect.left *= scale;
                        tv_srcrect.top *= scale;
                        tv_srcrect.right *= scale;
                        tv_srcrect.bottom *= scale;
                        tv_srcrect.offset((int) mPsiBmp.mLeftTop.x, (int) mPsiBmp.mLeftTop.y);
                        tv_srcrect.intersect(0, 0, mPsiBmp.mBmp.getWidth(), mPsiBmp.mBmp.getHeight());
                        tv_dstrect.set(tv_srcrect);
                        tv_dstrect.offset((int) -mPsiBmp.mLeftTop.x, (int) -mPsiBmp.mLeftTop.y);
                        tv_dstrect.left /= scale;
                        tv_dstrect.top /= scale;
                        tv_dstrect.right /= scale;
                        tv_dstrect.bottom /= scale;
                        canvas.drawBitmap(mPsiBmp.mBmp, tv_srcrect, tv_dstrect, mPaint);
                    }
                }
            }
        }
    }

    @Override
    protected void setPaintProperty(PDFViewCtrl pdfViewCtrl, int pageIndex, Paint paint) {
        paint.setColor(mColor);
        paint.setAlpha(AppDmUtil.opacity100To255(mOpacity));
        paint.setStrokeCap(Paint.Cap.ROUND);
        paint.setStrokeWidth(UIAnnotFrame.getPageViewThickness(pdfViewCtrl, pageIndex, mThickness));
    }

    @Override
    public long getSupportedProperties() {
        return mUtil.getSupportedProperties(true, mbHighlighter);
    }

    @Override
    protected void setPropertyBarProperties(PropertyBar propertyBar) {
        int[] colors = new int[PropertyBar.PB_COLORS_TOOL_DEFAULT.length];
        System.arraycopy(PropertyBar.PB_COLORS_TOOL_DEFAULT, 0, colors, 0, colors.length);
        colors[0] = PropertyBar.PB_COLORS_TOOL_DEFAULT[0];
        propertyBar.setColors(colors);
        propertyBar.setProperty(PropertyBar.PROPERTY_OPACITY, mOpacity);
        super.setPropertyBarProperties(propertyBar);
    }

    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        addAnnot();
    }

    private void addAnnot() {
        addAnnot(mCapturedPage, true, null);
    }

    int getCurrentPage() {
        return mCapturedPage;
    }

    public void addAnnot(boolean addUndo, final Event.Callback result) {
        addAnnot(mCapturedPage, addUndo, new Event.Callback() {
            @Override
            public void result(Event event, boolean success) {
                if (mUiExtensionsManager.getCurrentToolHandler() == InkToolHandler.this
                        && mPencilType == InkConstants.PENCIL_WITH_BRUSH) {
                    initPsiCanvas();
                }
                if (result != null) {
                    result.result(event, success);
                }
            }
        });
    }

    void addAnnot(int pageIndex, boolean addUndo, Event.Callback result) {
        addAnnot(pageIndex, addUndo, true, result);
    }

    private void addAnnot(final int pageIndex, final boolean addUndo, final boolean releasePsi, final Event.Callback result) {
        if (pageIndex == -1) {
            updateToolButtonStatus();
            clearUndoMembers(pageIndex);
            if (result != null)
                result.result(null, true);
            return;
        }

        ArrayList<ArrayList<PointF>> lineList = mLineListArray.get(pageIndex);
        if (lineList == null || lineList.size() == 0) {
            updateToolButtonStatus();
            clearUndoMembers(pageIndex);
            if (result != null)
                result.result(null, true);
            return;
        }

        if (mIsAddingAnnot) return;
        mIsAddingAnnot = true;

        float thickness = mThickness;
        ArrayList<Float> pressures = null;
        if (mPencilType == InkConstants.PENCIL_WITH_BRUSH) {
            pressures = new ArrayList<>();
            ArrayList<ArrayList<PSIData>> psiDatas = mPSIDataListArray.get(pageIndex);
            for (ArrayList<PSIData> psiDataArrayList : psiDatas) {
                for (PSIData psiData : psiDataArrayList) {
                    pressures.add(psiData.pressure);
                }
            }
            int diameter = thicknessToPsiDiameter(pageIndex, thickness);
            thickness = diameter < 1 ? 0.3f : thickness;
        }

        RectF bbox = new RectF();
        ArrayList<ArrayList<PointF>> docLines = mUtil.docLinesFromPageView(mPdfViewCtrl, pageIndex, lineList, bbox);
        bbox.inset(-thickness, -thickness);

        mAnnotHandler.addAnnot(mNM,
                mPencilType,
                pageIndex,
                new RectF(bbox),
                mColor,
                AppDmUtil.opacity100To255(mOpacity),
                thickness,
                docLines,
                pressures,
                true,
                false,
                new Event.Callback() {
                    @Override
                    public void result(Event event, boolean success) {
                        if (success) {
                            if (addUndo) {
                                ArrayList<IUndoItem> undoItems = mUndoItemsArray.get(pageIndex);
                                if (undoItems != null && undoItems.size() > 0)
                                    mUiExtensionsManager.getDocumentManager().addUndoItems(undoItems);

                                ArrayList<IUndoItem> redoItems = mRedoItemsArray.get(pageIndex);
                                if (redoItems != null && redoItems.size() > 0)
                                    mUiExtensionsManager.getDocumentManager().addRedoItems(redoItems);
                            }
                        }
                        release(pageIndex, releasePsi);
                        mIsAddingAnnot = false;

                        if (result != null) {
                            result.result(event, success);
                        }
                    }
                });
    }

    private void addUndoItem(final int pageIndex, final Event.Callback callback) {
        if (pageIndex == -1) {
            return;
        }
        final ArrayList<ArrayList<PointF>> lineList = mLineListArray.get(pageIndex);
        if (lineList == null || lineList.size() == 0) {
            callback.result(null, true);
            return;
        }
        mUiExtensionsManager.getDocumentManager().setDocModified(true);

        mDisposable = Single.fromCallable(new Callable<Boolean>() {
            @Override
            public Boolean call() throws Exception {
                _addUndoItem(pageIndex, lineList, callback);
                return true;
            }
        }).subscribeOn(Schedulers.io()).subscribe();
    }

    private void _addUndoItem(int pageIndex, ArrayList<ArrayList<PointF>> lineList, Event.Callback callback) {
        RectF bbox = new RectF();
        ArrayList<ArrayList<PointF>> docLines = mUtil.docLinesFromPageView(mPdfViewCtrl, pageIndex, lineList, bbox);
        bbox.inset(-mThickness, -mThickness);

        if (docLines.size() == 1) {
            mNM = AppDmUtil.randomUUID(null);

            final InkAddUndoItem undoItem = new InkAddUndoItem(mAnnotHandler, mPdfViewCtrl);
            undoItem.mPageIndex = pageIndex;
            undoItem.mNM = mNM;
            undoItem.mBBox = new RectF(bbox);
            undoItem.mAuthor = ((UIExtensionsManager) mPdfViewCtrl.getUIExtensionsManager()).getAnnotAuthor();
            undoItem.mFlags = Annot.e_FlagPrint;
            undoItem.mSubject = InkAnnotHandler.SUBJECT;
            undoItem.mCreationDate = AppDmUtil.currentDateToDocumentDate();
            undoItem.mModifiedDate = AppDmUtil.currentDateToDocumentDate();
            undoItem.mColor = mColor;
            undoItem.mOpacity = AppDmUtil.opacity100To255(mOpacity) / 255f;
            undoItem.mLineWidth = mThickness;
            undoItem.mPencilType = mPencilType;
            undoItem.mPath = InkAnnotUtil.pointsToPath(docLines);
            undoItem.mInkLists = InkAnnotUtil.cloneInkList(docLines);

            if (mPencilType == InkConstants.PENCIL_WITH_BRUSH) {
                ArrayList<Float> pressures = new ArrayList<>();
                ArrayList<ArrayList<PSIData>> psiDatas = mPSIDataListArray.get(pageIndex);
                for (ArrayList<PSIData> psiDataArrayList : psiDatas) {
                    for (PSIData psiData : psiDataArrayList) {
                        pressures.add(psiData.pressure);
                    }
                }
                undoItem.mPSIPressures = pressures;
            }

            ArrayList<IUndoItem> undoItems = mUndoItemsArray.get(pageIndex);
            if (undoItems == null) {
                undoItems = new ArrayList<>();
                mUndoItemsArray.put(pageIndex, undoItems);
            }
            undoItems.add(undoItem);

            ArrayList<IUndoItem> redoItems = mRedoItemsArray.get(pageIndex);
            if (redoItems != null)
                redoItems.clear();
        } else {
            ArrayList<ArrayList<PointF>> newLines = InkAnnotUtil.cloneInkList(docLines);
            docLines.remove(docLines.size() - 1);
            ArrayList<ArrayList<PointF>> oldLines = InkAnnotUtil.cloneInkList(docLines);

            InkModifyUndoItem undoItem = new InkModifyUndoItem(mAnnotHandler, mPdfViewCtrl);
            undoItem.mPageIndex = pageIndex;
            undoItem.mColor = mColor;
            undoItem.mBBox = new RectF(bbox);
            undoItem.mNM = mNM;
            undoItem.mOpacity = AppDmUtil.opacity100To255(mOpacity) / 255f;
            undoItem.mLineWidth = mThickness;
            undoItem.mModifiedDate = AppDmUtil.currentDateToDocumentDate();
            undoItem.mInkLists = newLines;
            undoItem.mPencilType = mPencilType;
            undoItem.mPath = InkAnnotUtil.pointsToPath(newLines);

            undoItem.mOldModifiedDate = AppDmUtil.currentDateToDocumentDate();
            undoItem.mOldBBox = new RectF(bbox);
            undoItem.mOldColor = mColor;
            undoItem.mOldOpacity = AppDmUtil.opacity100To255(mOpacity) / 255f;
            undoItem.mOldLineWidth = mThickness;
            undoItem.mOldInkLists = oldLines;
            undoItem.mOldPath = InkAnnotUtil.pointsToPath(oldLines);

            ArrayList<IUndoItem> undoItems = mUndoItemsArray.get(pageIndex);
            if (undoItems == null) {
                undoItems = new ArrayList<>();
                mUndoItemsArray.put(pageIndex, undoItems);
            }
            undoItems.add(undoItem);

            ArrayList<IUndoItem> redoItems = mRedoItemsArray.get(pageIndex);
            if (redoItems != null)
                redoItems.clear();
        }

        if (callback != null) {
            callback.result(null, true);
        }
    }

    private IToolSupply mToolSupply;
    private ToolItemBean mCurToolItem;
    private PropertyBar.PropertyChangeListener mCustomPropertyListener;

    public ToolItemBean getCurToolItem() {
        return mCurToolItem;
    }

    PropertyBar.PropertyChangeListener getCustomPropertyListener() {
        return mCustomPropertyListener;
    }

    IToolSupply getToolSupply() {
        if (mToolSupply == null)
            mToolSupply = new InkToolSupply(mContext);
        return mToolSupply;
    }

    public void deSelect() {
        ArrayList<ArrayList<PointF>> lineList = mLineListArray.get(mCapturedPage);
        if (lineList != null && !lineList.isEmpty())
            lineList.clear();
    }

    private class InkToolSupply extends ToolSupplyImpl {

        public InkToolSupply(Context context) {
            super(context);
        }

        @Override
        public int getToolBackgroundResource(int toolType) {
            if (toolType == ToolConstants.Pencil)
                return R.drawable.drawing_tool_pencil_bg;
            else
                return R.drawable.drawing_tool_highlighter_bg;
        }

        @Override
        public int getToolForegroundResource(int toolType) {
            if (toolType == ToolConstants.Pencil)
                return R.drawable.drawing_tool_pencil_src;
            else
                return R.drawable.drawing_tool_highlighter_src;
        }

        @Override
        public ToolProperty createToolProperty(int toolType) {
            ToolProperty property = new ToolProperty();
            if (toolType == ToolConstants.Pencil) {
                PencilConfig config = mUiExtensionsManager.getConfig().uiSettings.annotations.pencil;
                property.type = ToolConstants.Pencil;
                property.color = config.color;
                property.opacity = (int) (config.opacity * 100);
                property.lineWidth = config.thickness;
                property.style = InkConstants.PENCIL_WITH_PEN;
            } else {
                property.type = ToolConstants.Highlighter;
                property.color = PropertyBar.PB_COLORS_TOOL_DEFAULT[1];
                property.opacity = 50;
                property.lineWidth = 12f;
                property.style = InkConstants.PENCIL_WITH_PEN;
            }
            return property;
        }

        @Override
        public String getToolName(int toolType) {
            return JsonConstants.TYPE_INK;
        }

        @Override
        public void onClick(ToolItemBean itemBean) {
            mCurToolItem = itemBean;
            if (itemBean.toolItem.isSelected()) {
                mbHighlighter = itemBean.type == ToolConstants.Highlighter;
                if (mUiExtensionsManager.getMainFrame().getCurrentTab() == ToolbarItemConfig.ITEM_DRAWING_TAB) {
                    if (mbHighlighter) {
                        mUiExtensionsManager.onUIInteractElementClicked(IUIInteractionEventListener.Reading_DrawingBar_Highlight);
                    } else {
                        mUiExtensionsManager.onUIInteractElementClicked(IUIInteractionEventListener.Reading_DrawingBar_Pencil);
                    }
                }
                ToolProperty property = itemBean.property;
                if (property == null) {
                    property = createToolProperty(itemBean.type);
                    itemBean.property = property;
                }
                mPencilType = property.style;
                mColor = property.color;
                mOpacity = property.opacity;
                mThickness = property.lineWidth;
                mUiExtensionsManager.setCurrentToolHandler(InkToolHandler.this);
            } else {
                if (mUiExtensionsManager.getCurrentToolHandler() == InkToolHandler.this) {
                    mbHighlighter = false;
                    mCurToolItem = null;
                    mUiExtensionsManager.setCurrentToolHandler(null);
                }
            }
        }

        @Override
        public void resetPropertyBar(ToolItemBean itemBean, PropertyBar.PropertyChangeListener propertyChangeListener) {
            mbHighlighter = itemBean.type == ToolConstants.Highlighter;
            mCustomPropertyListener = propertyChangeListener;
            mCurToolItem = itemBean;

            ToolProperty property = itemBean.property;
            if (property == null) {
                property = createToolProperty(itemBean.type);
                itemBean.property = property;
            }
            mColor = property.color;
            mOpacity = property.opacity;
            mThickness = property.lineWidth;
            mPencilType = property.style;

            InkToolHandler.this.resetPropertyBar();
            mPropertyBar.setDismissListener(new PropertyBar.DismissListener() {
                @Override
                public void onDismiss() {
                    mPropertyBar.setDismissListener(null);
                    mCurToolItem = null;
                    mCustomPropertyListener = null;
                }
            });
        }

        @Override
        public PropertyBar getPropertyBar() {
            return mPropertyBar;
        }
    }

    void initPsiCanvas() {
        try {
            Rect outRect = new Rect();
            mUiExtensionsManager.getRootView().getWindowVisibleDisplayFrame(outRect);
            mPsi = new PSI(outRect.width(), outRect.height(), false);
            mPsi.setColor(mColor);
            mPsi.setOpacity((float) mOpacity / 100);
            int pageIndex = mCapturedPage == -1 ? mPdfViewCtrl.getCurrentPage() : mCapturedPage;
            mPsi.setDiameter(getPsiDiameter(pageIndex, mThickness));
        } catch (PDFException e) {
            e.printStackTrace();
        }
    }

    private int getPsiDiameter(int pageIndex, float thickness) {
        return Math.max(1, thicknessToPsiDiameter(pageIndex, thickness));
    }

    private int thicknessToPsiDiameter(int pageIndex, float thickness) {
        return (int) (UIAnnotFrame.getPageViewThickness(mPdfViewCtrl, pageIndex, thickness) * 1.2419);
    }

    private void releasePsiCanvas() {
        mPsiBmp.release();
        if (mPsi != null) {
            mPsi.delete();
            mPsi = null;
        }
    }

    private void setPSIColor(int color) {
        if (mPsi == null)
            initPsiCanvas();
        try {
            mPsi.setColor(color);
        } catch (PDFException e) {
            e.printStackTrace();
        }
    }

    private void setPSIOpacity(int opacity) {
        if (mPsi == null)
            initPsiCanvas();
        try {
            mPsi.setOpacity((float) opacity / 100);
        } catch (PDFException e) {
            e.printStackTrace();
        }
    }

    private void setPSIThickness(float thickness) {
        if (mPsi == null)
            initPsiCanvas();
        try {
            int pageIndex = mCapturedPage == -1 ? mPdfViewCtrl.getCurrentPage() : mCapturedPage;
            mPsi.setDiameter(getPsiDiameter(pageIndex, thickness));
        } catch (PDFException e) {
            e.printStackTrace();
        }
    }

    static class PsiBitmap {
        Bitmap mBmp;
        int mPage;
        PointF mLeftTop;
        PointF mPvSize;
        long mToken;

        void release() {
            if (mBmp != null && !mBmp.isRecycled()) {
                mBmp.recycle();
                mBmp = null;
            }
            mPage = -1;
            mLeftTop = null;
            mPvSize = null;
        }
    }

    private void release(int pageIndex, boolean releasePsi) {
        mLineListArray.remove(pageIndex);
        mPathListArray.remove(pageIndex);
        mUndoItemsArray.remove(pageIndex);
        mPSIDataListArray.remove(pageIndex);
        clearUndoMembers(pageIndex);

        if (releasePsi)
            releasePsiCanvas();
//        mCapturedPage = -1;
    }

    private float calibrationPressure(float pressure) {
        if (pressure <= 1)
            return pressure;

        int powerNumber = String.valueOf((int) Math.floor(pressure)).length();
        if (pressure % 10 == 0) {
            powerNumber -= 1;
        }
        return (float) (pressure / Math.pow(10, powerNumber));
    }

    private Handler handler;
    private Runnable timeoutRunnable;

    private void handleInkTimeOut() {
        cancelInkTimeOut();

        timeoutRunnable = new Runnable() {
            @Override
            public void run() {
                addAnnot(mCapturedPage, true, new Event.Callback() {
                    @Override
                    public void result(Event event, boolean success) {
                        if (mPencilType == InkConstants.PENCIL_WITH_BRUSH)
                            initPsiCanvas();
                    }
                });
            }
        };

        if (handler == null) {
            handler = new Handler(Looper.getMainLooper());
        }
        handler.postDelayed(timeoutRunnable, mInkTimeout);
    }

    private void cancelInkTimeOut() {
        if (timeoutRunnable != null) {
            handler.removeCallbacks(timeoutRunnable);
        }
    }

}
